/*
 * $Id: 2d229ac872a502797d7bdd50597d6d21ffa4666b $
 *
 * This file is part of ntldap.
 *
 * Copyright (c) 2006, 2007, 2008 Niklas Olmes <niklas@noxa.de>
 *                                http://www.nevm.de/ntldap/
 *                                NEVM-RIPE
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License Version 2
 * as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software Foundation,
 * Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 */

#include <stdio.h>
#include <stdlib.h>

#include "cdk/cdk.h"

#include "ntg.h"
#include "ntg_cdkmenu.h"
#include "nt_alloc.h"

static const char rcsid[] = "$Id: 2d229ac872a502797d7bdd50597d6d21ffa4666b $";

#define S()			ctx->cdkmenus[n] = nt_calloc(1, sizeof(NTG_CDKMENU)); ctx->cdkmenus[n]->numentries = 0; cur_x = -1;
#define E()			n++;

#define X(name, pos)		cur_x++; cur_y = 0; \
				pe = ctx->cdkmenus[n]->entries[cur_x]; pi = ctx->cdkmenus[n]->info[cur_x]; pm = ctx->cdkmenus[n]->mapping[cur_x]; \
				pe[0] = nt_strdup(name); pi[0] = nt_strdup(""); pm[0] = 0; \
				ctx->cdkmenus[n]->loc[cur_x] = pos; \
				(ctx->cdkmenus[n]->numentries)++; \
				ctx->cdkmenus[n]->numsubs[cur_x] = 1;
#define Y(name, info, map)	cur_y++; pe[cur_y] = nt_strdup(name); pi[cur_y] = nt_strdup(info); pm[cur_y] = map; \
				(ctx->cdkmenus[n]->numsubs[cur_x])++;
void
ntg_cdkmenu_init(NTG_CTX *ctx)
{
	char **pe, **pi;
	int *pm;
	int n, cur_x, cur_y;

	#define M_START()	S(); X("NOP", LEFT); Y("NOP", "Do nothing.", 0);
	#define M_END()		X("Help", RIGHT); Y("Keys", "Show key mappings.", 0); Y("About", "About ntLDAP.", 0); E();

	ctx->cdkmenus = nt_calloc(4, sizeof(NTG_CDKMENU *));

	n = 0;

	M_START();
	X("Navigation", LEFT);
	Y("Return (^X)", "Return to the previous screen.", NTG_MENU_RET_RETURN);
	Y("Exit (F10)", "Exit the program.", NTG_MENU_RET_SUICIDE);
	M_END();

	M_START();
	X("Navigation", LEFT);
	Y("Refine Search (;)", "Refine currently used search filter.", NTG_MENU_RET_REFSEARCH);
	Y("Subsearch (^/)", "Start a subsearch below current Base DN.", NTG_MENU_RET_SUBSEARCH);
	Y("Change Base (M-b)", "Change current Base DN.", NTG_MENU_RET_CHBASE);
	Y("Plaintext Search Left Side (/)", "Search for an ASCII string on the left column.", ctx->keys->search);
	Y("Plaintext Search DN (?)", "Search for an ASCII string on the DNs (right column).", ctx->keys->search_dn);
	Y("Display Current (Return)", "Display current item in generic form.", ctx->keys->display_auto);
	Y("Display Current With Template (Backsp)", "Display current item with predefined template.", ctx->keys->display_tpl);
	Y("Dive (^D)", "Dive into current items DN.", NTG_MENU_RET_DIVE);
	Y("Return (^X)", "Return to the previous screen.", NTG_MENU_RET_RETURN);
	Y("Exit (F10)", "Exit the program.", NTG_MENU_RET_SUICIDE);

	X("Operations", LEFT);
	Y("Refresh (^R)", "Refresh results.", NTG_MENU_RET_REFRESH);
	Y("Create via Template (M-T)", "Create new object from predefined template.", NTG_MENU_RET_CREATE_FROM_TPL);
	Y("Create via Schema (M-O)", "Create new object from arbitrary OC combinations.", NTG_MENU_RET_CREATE_FROM_OC);
	Y("Delete Current Object (M-d)", "Delete the current object.", NTG_MENU_RET_DEL);
	Y("Delete Selected Objects (M-D)", "Delete all selected objects.", NTG_MENU_RET_DELM);
	Y("Copy Current Object (M-c)", "Copy the current object to a specified Base DN.", NTG_MENU_RET_COPY);
	Y("Copy Selected Objects (M-C)", "Copy all selected objects to a specified Base DN.", NTG_MENU_RET_COPYM);
	Y("Move Current Object (M-m)", "Move the current object to a specified Base DN.", NTG_MENU_RET_MOVE);
	Y("Move Selected Objects (M-M)", "Move all selected objects to a specified Base DN.", NTG_MENU_RET_MOVEM);
	M_END();
}

void
ntg_cdkmenu_free(NTG_CDKMENU *menu)
{
	int i, j;

	if (menu == NULL)
		return;

	if (menu->entries != NULL)
		for (i = 0; i < menu->numentries; i++)
			for (j = 0; j < menu->numsubs[i]; j++) {
				free(menu->entries[i][j]);
				free(menu->info[i][j]);
			}
	free(menu);
}

int
ntg_cdkmenu_display(NTG_CTX *ctx, NTG_CDKMENU *m)
{
	CDKMENU *menu;
	char *msg[2];
	int sel;

	msg[0] = nt_strdup("                                                             ");
	msg[1] = NULL;
	m->label = newCDKLabel(ctx->cdkrootscreen, LEFT, BOTTOM, msg, 1, FALSE, FALSE);
	free(msg[0]);

	if ((menu = newCDKMenu(ctx->cdkrootscreen, m->entries, m->numentries, m->numsubs, m->loc, TOP, A_REVERSE, A_REVERSE)) == NULL)
		ntg_fail(ctx, "failed to create CDKMenu, should not happen", NTG_E_INTERN);
	setCDKMenuPreProcess(menu, ntg_cdkmenu_preprocess, m);
	setCDKMenuPostProcess(menu, ntg_cdkmenu_postprocess, m);

	sel = activateCDKMenu(menu, NULL);

	if (menu->exitType != vNORMAL)
		sel = -1; /* invalid */

	eraseCDKLabel(m->label);
	destroyCDKLabel(m->label);
	eraseCDKMenu(menu);
	destroyCDKMenu(menu);

	ntg_refresh(ctx);

	if (sel < 0 || sel / 100 >= m->numentries || sel % 100 >= m->numsubs[sel/100])
		return 0; /* impossible, bail */

	return m->mapping[sel/100][sel%100 + 1];
}

int
ntg_cdkmenu_preprocess(EObjectType cdktype, void *object, void *clientdata, chtype input)
{

	/* unused */
	(void)cdktype;
	(void)clientdata;
	(void)object;
	(void)input;

	return 1;
}

int
ntg_cdkmenu_postprocess(EObjectType cdktype, void *object, void *clientdata, chtype input)
{
	CDKMENU *menu;
	NTG_CDKMENU *m;
	char *msg[2];
	char buf[256];

	menu = object;
	m = clientdata;
	/* unused */
	(void)cdktype;
	(void)input;

	(void)snprintf(buf, sizeof(buf), "%s", m->info[menu->currentTitle][menu->currentSubtitle + 1]);
	msg[0] = buf;
	msg[1] = NULL;

	setCDKLabelMessage(m->label, msg, 1);
	drawCDKLabel(m->label, FALSE);

	return 0;
}
