/*
 * $Id: f07f6785fc3d03f1b156b718a2161e202e780154 $
 *
 * This file is part of ntldap.
 *
 * Copyright (c) 2006, 2007, 2008 Niklas Olmes <niklas@noxa.de>
 *                                http://www.nevm.de/ntldap/
 *                                NEVM-RIPE
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License Version 2
 * as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software Foundation,
 * Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 */

#ifndef HAD_LIBNTLDAP_NTL_H
#define HAD_LIBNTLDAP_NTL_H

#include <regex.h>

#include "ldap.h"

#define NTL_T_MAIL	1
#define NTL_R_MAIL	"^( |[a-zA-Z0-9][a-zA-Z0-9_.-]*@[a-zA-Z0-9][a-zA-Z0-9-]*\\.[a-zA-Z0-9][a-zA-Z0-9.-]*) *$"
#define NTL_T_NAME	2
#define NTL_R_NAME	"^( |[a-zA-Z][a-zA-Z]*) *$"
#define NTL_T_CN	3
#define NTL_R_CN	"^( |[a-zA-Z0-9][a-zA-Z0-9. -]*) *$"
#define NTL_T_INT	4
#define NTL_T_NUM	5
#define NTL_T_BOOL	6
#define NTL_T_ALPHA	7
#define NTL_T_ALNUM	8
#define NTL_T_IPV4	9
#define NTL_T_REGEX	10
#define NTL_T_ENUM	11

typedef struct ntl_attr {
	char *name;
	char **val;
	int tsize;
	int vsize;
} NTL_ATTR;

typedef struct ntl_tpl {
	char *desc;
	char *attr;
	int a;
	int type;
	int null;
	int multi;
	int prec;
	char *regex;
	char **en;
} NTL_TPL;

typedef struct ntl_tplc {
	char *name;
	char **oc;
	int ocmax;
	NTL_TPL **tpl;
	NTL_ATTR *crattr;
	NTL_ATTR *usrattr;
	int tplmax;
	int crmax;
	int usrmax;
} NTL_TPLC;

typedef struct ntl_cfg {
	char *dn;
	char *cred;
	char *basedn;
	char *host;
	long port;
	NTL_TPLC **tplc;
	int secure;
	int showop;
} NTL_CFG;

LDAP *ntl_connect(const NTL_CFG *);
LDAPMessage *ntl_search(LDAP *, const char *, int, const char *, const char **);

char **ntl_ldif_entry(const NTL_ATTR *);
char **ntl_ldif_attr(const NTL_ATTR);
void ntl_ldif_free(char **);

NTL_ATTR *ntl_entry(LDAP *, LDAPMessage *);
void ntl_entry_free(NTL_ATTR *);

const char **ntl_lookup(const NTL_ATTR *, const char *);
int ntl_lookup_val(const NTL_ATTR *, const char *, const char *);

int ntl_mod(LDAP *, const char *, const char *, const char **, int);
int ntl_add(LDAP *, const char *, const NTL_ATTR *);
int ntl_copy_base(LDAP *, const char *, const char *);
int ntl_copy(LDAP *, const char *, const char *);
int ntl_del(LDAP *, const char *);
int ntl_move_base(LDAP *, const char *, const char *);
int ntl_move(LDAP *, const char *, const char *);

void ntl_attr_add(NTL_ATTR *, const char *, const char *);
void ntl_attr_add_nocheck(NTL_ATTR *, const char *, const char *);

char *ntl_strip(const char *);
void ntl_in_nl(char *);
void ntl_out_nl(char *);

char **ntl_ldap_err(const char *, const char *, LDAP *, int);
void ntl_ldap_err_free(char **);

#endif
