/*
 * $Id: f386dd6a396aa1ac2c44560b9ecd5f4ccc0de23f $
 *
 * This file is part of ntldap.
 *
 * Copyright (c) 2006, 2007, 2008 Niklas Olmes <niklas@noxa.de>
 *                                http://www.nevm.de/ntldap/
 *                                NEVM-RIPE
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License Version 2
 * as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software Foundation,
 * Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 */

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <time.h>

#include "ntu.h"
#include "nt_alloc.h"

static const char rcsid[] = "$Id: f386dd6a396aa1ac2c44560b9ecd5f4ccc0de23f $";

/* 0 ... 63 => ascii - 64 */
static const char itoa64[] = "./0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz";
static int ntu_hadseed = 0;

int
ntu_strcmp(const void *p1, const void *p2)
{
	return strcmp(*(char **)p1, *(char **) p2);
}

int
ntu_strcasecmp(const void *p1, const void *p2)
{
	return strcasecmp(*(char **)p1, *(char **) p2);
}

void
ntu_to64(char *s, long v, int n)
{
	while (--n >= 0) {
		*s++ = itoa64[v & 0x3f]; /* 0x3f => 63 */
		v >>= 6;
	}
}

void
ntu_seed(void)
{
	FILE *fp;
	unsigned long seed;

	if (ntu_hadseed)
		return;

	if ((fp = fopen("/dev/urandom", "r")) == NULL
		&& (fp = fopen("/dev/prandom", "r")) == NULL) {
		seed = 134907;
		seed += gethostid();
		seed ^= getpid();
		seed ^= getppid();
	} else {
		(void)fread(&seed, sizeof(seed), 1, fp);
		(void)fclose(fp);
	}

	srandom(seed);
	ntu_hadseed = 1;
}

char *
ntu_gentime(time_t tloc)
{
	struct tm *t;
	char buf[64];

	if ((t = gmtime(&tloc)) == NULL)
		return NULL;

	*buf = '\0';
	if (strftime(buf, sizeof(buf), "%Y%m%d%H%MZ", t) == 0)
		return NULL;

	return nt_strdup(buf);
}

char *
ntu_midtime(time_t tloc)
{
	struct tm *t;
	char buf[64];

	if ((t = gmtime(&tloc)) == NULL)
		return NULL;

	*buf = '\0';
	if (strftime(buf, sizeof(buf), "%Y%m%d%H%M%S", t) == 0)
		return NULL;

	return nt_strdup(buf);
}
